/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.ClientRepresentation;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

class KeycloakClientMapperTest {

	@Spy
	private final KeycloakClientMapper mapper = Mappers.getMapper(KeycloakClientMapper.class);

	@Nested
	class TestMap {

		@Test
		void shouldMapClientId() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getClientId()).isEqualTo(OzgKeycloakClientSpecTestFactory.CLIENT_ID);
		}

		@Test
		void shouldMapBaseUrl() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getBaseUrl()).isEqualTo(OzgKeycloakClientSpecTestFactory.BASE_URL);
		}

		@Test
		void shouldMapRedirectUris() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getRedirectUris()).isEqualTo(OzgKeycloakClientSpecTestFactory.REDIRECT_URLS);
		}

		@Test
		void shouldMapWebOrigins() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getWebOrigins()).isEqualTo(OzgKeycloakClientSpecTestFactory.WEB_ORIGINS);
		}

		@Test
		void testProtocol() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getProtocol()).isEqualTo(KeycloakClientMapper.PROTOCOL);
		}

		@Test
		void testImplicitFlowEnabled() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isImplicitFlowEnabled()).isTrue();
		}

		@Test
		void testDirectAccessGrantsEnabled() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isDirectAccessGrantsEnabled()).isTrue();
		}

		@Test
		void testStandardFlowEnabled() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isStandardFlowEnabled()).isTrue();
		}

		@Test
		void testPublicClientEnabled() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isPublicClient()).isTrue();
		}

		@Test
		void testBearerOnlyEnabled() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isBearerOnly()).isFalse();
		}

		@Test
		void testPostLogoutRedirectUri() {
			var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getAttributes())
					.containsEntry(KeycloakClientMapper.ATTRIBUTE_NAME_POST_LOGOUT_REDIRECT_URIS,
							KeycloakClientMapper.ATTRIBUTE_VALUE_POST_LOGOUT_REDIRECT_URIS);
		}
	}

	@Nested
	class TestUpdate {

		@Test
		void shouldMapClientId() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getClientId()).isEqualTo(OzgKeycloakClientSpecTestFactory.CLIENT_ID);
		}

		@Test
		void shouldMapBaseUrl() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getBaseUrl()).isEqualTo(OzgKeycloakClientSpecTestFactory.BASE_URL);
		}

		@Test
		void shouldMapRedirectUris() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getRedirectUris()).isEqualTo(OzgKeycloakClientSpecTestFactory.REDIRECT_URLS);
		}

		@Test
		void shouldMapWebOrigins() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getWebOrigins()).isEqualTo(OzgKeycloakClientSpecTestFactory.WEB_ORIGINS);
		}

		@Test
		void testProtocol() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getProtocol()).isEqualTo(KeycloakClientMapper.PROTOCOL);
		}

		@Test
		void testImplicitFlowEnabled() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isImplicitFlowEnabled()).isTrue();
		}

		@Test
		void testDirectAccessGrantsEnabled() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isDirectAccessGrantsEnabled()).isTrue();
		}

		@Test
		void testStandardFlowEnabled() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isStandardFlowEnabled()).isTrue();
		}

		@Test
		void testPublicClientEnabled() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isPublicClient()).isTrue();
		}

		@Test
		void testBearerOnlyEnabled() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.isBearerOnly()).isFalse();
		}

		@Test
		void testPostLogoutRedirectUri() {
			var client = mapper.update(new ClientRepresentation(), OzgKeycloakClientSpecTestFactory.create());

			assertThat(client.getAttributes())
					.containsEntry(KeycloakClientMapper.ATTRIBUTE_NAME_POST_LOGOUT_REDIRECT_URIS,
							KeycloakClientMapper.ATTRIBUTE_VALUE_POST_LOGOUT_REDIRECT_URIS);
		}
	}

	@Nested
	class TestMapClientRole {

		@Test
		void shouldMapClientName() {
			var role = mapper.mapRole(OzgKeycloakClientSpecTestFactory.ROLE1);

			assertThat(role.getName()).isEqualTo(OzgKeycloakClientSpecTestFactory.ROLE_NAME_1);
		}
	}

	@Nested
	class TestUpdateRole {

		@Test
		void shouldUpdateClientName() {
			var role = mapper.updateRole(RoleRepresentationTestFactory.create(), OzgKeycloakClientSpecTestFactory.ROLE2);

			assertThat(role.getName()).isEqualTo(OzgKeycloakClientSpecTestFactory.ROLE_NAME_2);
		}
	}

	@Nested
	class TestProtocolMapper {

		@Nested
		class TestMapProtocolMapper {

			@Test
			void shouldMapProtocolMapper() {
				var client = mapper.map(OzgKeycloakClientSpecTestFactory.create());

				assertThat(client.getProtocolMappers()).isNotEmpty();
			}

			@Test
			void shouldMapName() {
				var clientSpec = OzgKeycloakClientSpecTestFactory.create();

				var protocolMapper = mapper.map(clientSpec).getProtocolMappers().get(0);

				assertThat(protocolMapper.getName()).isEqualTo(OzgKeycloakClientSpecProtocolMapperTestFactory.NAME);
			}

			@Test
			void shouldMapProtocol() {
				var clientSpec = OzgKeycloakClientSpecTestFactory.create();

				var protocolMapper = mapper.map(clientSpec).getProtocolMappers().get(0);

				assertThat(protocolMapper.getProtocol()).isEqualTo(OzgKeycloakClientSpecProtocolMapperTestFactory.PROTOCOL);
			}

			@Test
			void shouldMapType() {
				var clientSpec = OzgKeycloakClientSpecTestFactory.create();

				var protocolMapper = mapper.map(clientSpec).getProtocolMappers().get(0);

				assertThat(protocolMapper.getProtocolMapper()).isEqualTo(OzgKeycloakClientSpecProtocolMapperTestFactory.TYPE);
			}

			@Test
			void shouldMapConfig() {
				var clientSpec = OzgKeycloakClientSpecTestFactory.create();

				var protocolMapper = mapper.map(clientSpec).getProtocolMappers().get(0);

				assertThat(protocolMapper.getConfig()).containsEntry(OzgKeycloakClientSpecProtocolMapperTestFactory.CONFIG_KEY_1,
						OzgKeycloakClientSpecProtocolMapperTestFactory.CONFIG_VALUE_1);
			}
		}
	}
}
