/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;
import de.ozgcloud.operator.keycloak.user.OzgKeycloakUserSpec.KeycloakUserSpecUser;
import io.javaoperatorsdk.operator.api.reconciler.DeleteControl;

class KeycloakUserReconcilerTest {

	@Spy
	@InjectMocks
	private KeycloakUserReconciler reconciler;

	@Mock
	private KeycloakUserService service;

	@Mock
	private KeycloakUserPreconditionService preconditionService;

	@Nested
	class TestAddUser {

		@BeforeEach
		void init() {
			when(preconditionService.getPreconditionErrors(any())).thenReturn(Optional.empty());
		}

		@Test
		void shouldCallServiceAddUser() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

			reconciler.reconcile(user, null);

			verify(service).createOrUpdateUser(user.getSpec(), OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldUpdateStatus() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

			reconciler.reconcile(user, null);

			assertThat(user.getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.OK);
		}

		@Nested
		class TestPreconditionError {

			private static final String ERROR_MESSAGE = "ErrorMessage";

			@Test
			void shouldSetPreconcitionErrorMessage() {
				when(preconditionService.getPreconditionErrors(any())).thenReturn(Optional.of(ERROR_MESSAGE));
				OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

				reconciler.reconcile(user, null);

				assertThat(user.getStatus().getMessage()).isEqualTo(ERROR_MESSAGE);
			}
		}
	}

	@Nested
	@Disabled("ErrorStatusHandler erstmal deaktiviert weil der nicht funktioniert hat")
	class TestErrorStatusUpdateControl {

		@Test
		void shouldSetErrorStatusOnException() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();
			doThrow(new RuntimeException()).when(service).createOrUpdateUser(any(), any());

			reconciler.reconcile(user, null);

			assertThat(user.getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.ERROR);
		}
	}

	@DisplayName("Cleanup")
	@Nested
	class TestCleanup {

		@DisplayName("with keep_after_delete")
		@Nested
		class TestWithKeepAfterDelete {

			private final KeycloakUserSpecUser keycloakUser = KeycloakUserSpecUserTestFactory.createBuiler().build();
			private final OzgKeycloakUserSpec userSpec = OzgKeycloakUserSpecTestFactory.createBuilder().keepAfterDelete(true)
					.keycloakUser(keycloakUser).build();
			private final OzgKeycloakUser user = OzgKeycloakUserTestFactory.create(userSpec);

			@Test
			void shouldNotCallService() {
				reconciler.cleanup(user, null);

				verify(service, never()).deleteUser(userSpec, OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
			}

			@Test
			void shouldReturnDeleteControl() {
				var control = reconciler.cleanup(user, null);

				assertThat(control).usingRecursiveComparison().isEqualTo(DeleteControl.defaultDelete());
			}
		}

		@Test
		void shouldCallServiceDelete() {
			var user = OzgKeycloakUserTestFactory.create();

			reconciler.cleanup(user);

			verify(service).deleteUser(user.getSpec(), OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
		}
	}
}
