package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;

class UserSecretServiceTest {

	private final static String NAMESPACE = "dummyNamespace";

	@Spy
	@InjectMocks
	private UserSecretService userSecretService;
	@Mock
	private UserNameConverter userNameConverter;
	@Mock
	private UserSecretBuilder secretBuilder;
	@Mock
	private UserSecretReader secretReader;
	@Mock
	private KubernetesRemoteService kubernetesRemoteService;

	@DisplayName("Create Secret")
	@Nested
	class TestCreateSecret {

		private final OzgKeycloakUserSpec userSpec = OzgKeycloakUserSpecTestFactory.create();

		@Mock
		private Resource<Secret> secretResource;
		@Mock
		private Secret secret;
		@Mock
		private ResourceAdapter<Secret> resourceAdapter;

		@BeforeEach
		void mock() {
			doReturn(secret).when(secretBuilder).build(any(), any(), any());
			doReturn(secretResource).when(userSecretService).getUserSecret(any(), any());
			doReturn(resourceAdapter).when(userSecretService).createResourceAdpater(any());
		}

		@Test
		void shouldConvertName() {
			userSecretService.create(userSpec, NAMESPACE);

			verify(userNameConverter).toSecretName(userSpec.getKeycloakUser());
		}

		@Test
		void shouldBuildUserSecret() {
			userSecretService.create(userSpec, NAMESPACE);

			verify(secretBuilder).build(any(), eq(userSpec.getKeycloakUser()), eq(NAMESPACE));
		}

		@Test
		void shouldGetUserSecret() {
			userSecretService.create(userSpec, NAMESPACE);

			verify(userSecretService).getUserSecret(userSpec, NAMESPACE);
		}

		@Test
		void shouldCreateResourcAdpater() {
			userSecretService.create(userSpec, NAMESPACE);

			verify(userSecretService).createResourceAdpater(secretResource);
		}

		@Test
		void shouldCreateSecret() {
			userSecretService.create(userSpec, NAMESPACE);

			verify(resourceAdapter).create(secret);
		}

		@Test
		void shouldReturnCreatedSecret() {
			Secret createdSecret = userSecretService.create(userSpec, NAMESPACE);

			assertThat(createdSecret).isNotNull();
		}
	}

	@DisplayName("Get password from secret")
	@Nested
	class TestGetPasswordFromSecret {

		@Test
		void shouldCallSecretReader() {
			Secret secret = SecretTestFactory.create();

			userSecretService.getPasswordFromSecret(secret);

			verify(secretReader).getPasswortFromSecret(secret);
		}
	}

	@DisplayName("Get user secret")
	@Nested
	class TestGetUserSecret {

		private final OzgKeycloakUserSpec userSpec = OzgKeycloakUserSpecTestFactory.create();

		private final static String CONVERTED_NAME = "SecretConformName";

		@BeforeEach
		void mockConverter() {
			when(userNameConverter.toSecretName(any())).thenReturn(CONVERTED_NAME);
		}

		@Test
		void shouldConvertToSecretName() {
			userSecretService.getUserSecret(userSpec, NAMESPACE);

			verify(userNameConverter).toSecretName(userSpec.getKeycloakUser());
		}

		@Test
		void shouldGetSecret() {
			userSecretService.getUserSecret(userSpec, NAMESPACE);

			verify(kubernetesRemoteService).getSecret(NAMESPACE, CONVERTED_NAME);
		}
	}

	@DisplayName("Get or create cluster secret")
	@Nested
	class TestGetOrCreateClusterSecret {

		@Mock
		private Resource<Secret> secretResource;

		private Secret secret = SecretTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(secretResource).when(userSecretService).getUserSecret(any(OzgKeycloakUserSpec.class), eq(NAMESPACE));
		}

		@Test
		void shouldReturnExistingSecret() {
			when(secretResource.get()).thenReturn(secret);

			Secret secretResponse = userSecretService.getOrCreateClusterSecret(OzgKeycloakUserSpecTestFactory.create(), NAMESPACE);

			assertThat(secretResponse).isSameAs(secret);
		}

		@Test
		void shouldReturnNewSecretIfNotExisting() {
			when(secretResource.get()).thenReturn(null);
			doReturn(secret).when(userSecretService).create(any(OzgKeycloakUserSpec.class), eq(NAMESPACE));

			Secret secretResponse = userSecretService.getOrCreateClusterSecret(OzgKeycloakUserSpecTestFactory.create(), NAMESPACE);

			assertThat(secretResponse).isSameAs(secret);
		}
	}
}